{% for post in posts %}
## {{ post.title }}

<div class="post-extra">
    <div class="col">
        <p class="post-date">{{ post.date }}</p>
    </div>
    <div class="col">
    {% for tag in post.tags %}
        <a href="{{ site_url }}{{ config.slug }}/{{ config.tags_subdir }}/{{ tag }}/">#{{ tag }}</a>
    {% endfor %}
    </div>
</div>

{{ post.teaser }}

{{ post.is_short }}


<div class="post-link">
{% if post.is_teaser %}
    <a href="{{ site_url }}/{{ config.slug }}/{{ post.slug }}/">{{ translation.teaser_link_text }}</a>
{% else %}
    &nbsp;
{% endif %}
</div>

{% endfor %}
